package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.shirotoken.AdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.SaAdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.pojo.platform.repository.SysUserRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @Description : 用户登陆方法
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-13 17:18
 * @Modify:
 **/
@Service
public class SystemLoginService implements ISystemLoginService {
    private static final Logger LOGGER = LoggerFactory.getLogger(SystemLoginService.class);

    @Autowired
    private SysUserRepository sysUserRDao;

    /**
     * 核心的缓存
     */
    @Resource(name="redisCore")
    protected ImppRedis redisCore;

    @Override
    public SessionUser queryUserLogin(String loginName, String loginPwd, String languageCode) {
        //生成令牌
        UserToken ut = new UserToken(loginName,loginPwd,languageCode);
        //token验证
        return AuthUtil.login(ut);
    }

    @Override
    public SessionUser queryAdminLogin(String loginName, String loginPwd, String languageCode) {
        //生成令牌
        AdminToken adminToken = new AdminToken(loginName,loginPwd,languageCode);
        //token验证
        return AuthUtil.login(adminToken);
    }

    @Override
    public SessionUser queryCheckUserLogin(UserToken userToken) {
        //验证用户
        SysUser user = getUserLoginInfo(userToken.getLoginName(),userToken.getLoginPwd());
        LOGGER.info("【验证用户checkUserLogin】{}",user);
        //未抛异常，封装用户
        return packSessionUser(user,CommonEnumUtil.USER_TYPE.USER.getCode(),userToken.getLanguageCode());
    }

    @Override
    public SessionUser queryCheckAdminLogin(AdminToken adminToken) {
        //验证用户
        SysUser user = getUserLoginInfo(adminToken.getLoginName(),adminToken.getLoginPwd());
        //未抛异常，封装用户
        return packSessionUser(user,CommonEnumUtil.USER_TYPE.ADMIN.getCode(),adminToken.getLanguageCode());
    }

    @Override
    public SessionUser queryCheckSaAdminLogin(SaAdminToken saAdminToken) {
        //SysUser user = getUserLoginInfo(saAdminToken.getLoginName(),saAdminToken.getLoginPwd());
        SysUser user = null;
        /**
         * to do
         *
         * 此处外网获取sa超级管理员用户名密码
         * 或者导入加密证书生成内存超级管理员用户名密码
         */
        return packSessionUser(user,CommonEnumUtil.USER_TYPE.SA.getCode(),saAdminToken.getLanguageCode());
    }

    @Override
//    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public Integer doLoginPasswordError(SysUser user) {
        String redisKey = CommonConstWords.USER_LOGIN_ERROR + "_" + user.getId();
//        Object redisValue = redisCore.getObject(redisKey);
        Object redisValue = AuthUtil.getSessionAttribute(redisKey);
        Integer num = redisValue == null ? 1 : Integer.parseInt(redisValue.toString()) + 1;


        if(num >= CommonConstWords.USER_LOGIN_ERROR_MAX_NUM){
            user.setUserStatus(CommonEnumUtil.USER_STATUS.LOCKING.getValue());
            redisCore.putObject(redisKey,num,0); // 账号锁定后移除锁
            sysUserRDao.update(user);
        }

        redisCore.putObject(redisKey,num,24 * 60 * 60);
        AuthUtil.setSessionObject(redisKey,num);
        return num;
    }

    /**
     * 根据用户名密码查询用户
     * @param loginName
     * @param pwd
     * @return
     */
    public SysUser getUserLoginInfo(String loginName,String pwd) {
        SysUser user = sysUserRDao.getByProperty("userLoginName",loginName);
        if(user == null){    //用户不存在
            throw new AuthenticationException("用户不存在");
        }else {
            if(user.getUserStatus() != CommonEnumUtil.USER_STATUS.ENABLE.getValue()){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.LOGIN_USER_NAME_EXCEPTION.getCode())
                        .setErrorDetail("账号状态:"+CommonEnumUtil.USER_STATUS.valueOf(user.getUserStatus()))
                        .setErrorSolution("请联系管理员")
                        .build();
            }
            try {
                // 密码加密
                pwd = EncryptTool.hexMD5(pwd);
            } catch (Exception e) {
                LOGGER.error("登录密码加密出错。");
                throw new AuthenticationException("登陆密码错误");
//                doLoginPasswordError( user);
//                throw new AuthenticationException("密码输入错误。如果输错次数超过"+CommonConstWords.USER_LOGIN_ERROR_MAX_NUM+"次，用户将被锁定。");
            }
            if (!StringUtils.equals(user.getUserLoginPassword(), pwd)) {    //密码不符
                throw new AuthenticationException("登陆密码错误");
//                doLoginPasswordError( user);
//                throw new AuthenticationException("密码输入错误。如果输错次数超过"+CommonConstWords.USER_LOGIN_ERROR_MAX_NUM+"次，用户将被锁定。");
            }

            return user;
        }
    }

    /**
     * 封装用户登陆信息
     * @param user
     * @param userType
     * @param languageCode
     * @return
     */
    private SessionUser packSessionUser(SysUser user,String userType, String languageCode) {
        SessionUser sessionUser = new SessionUser();
        sessionUser.setLanguageCode(languageCode);
//        sessionUser.setSiteCode(user.getFactoryCode());
        sessionUser.setUserName(user.getUserName());
        sessionUser.setUserType(userType);
//        sessionUser.setUserCode(user.getUserInfo().getLanguageCode());

        return sessionUser;
    }


}
